package com.fanruan.cloud.s3.shared.config;

import com.fr.config.ConfigContext;
import com.fr.config.DefaultConfiguration;
import com.fr.config.Identifier;
import com.fr.config.Status;
import com.fr.config.Visualization;
import com.fr.config.holder.Conf;
import com.fr.config.holder.factory.Holders;
import com.fr.intelli.record.Focus;
import com.fr.record.analyzer.EnableMetrics;
import com.fr.stable.StringUtils;

/**
 * @author richie
 * @version 10.0
 * Created by richie on 2020/6/18
 * 共享的S3配置，包括Endpoint、accessKey、accessSecretKey
 */
@Visualization(category = "Plugin-S3Cloud_Shared_Config")
@SuppressWarnings("all")
@EnableMetrics
public class S3CloudSharedConfig extends DefaultConfiguration {

    private static final String PASSWORD_HOLDER = "*************";

    private static volatile S3CloudSharedConfig config = null;

    public static S3CloudSharedConfig getInstance() {
        if (config == null) {
            config = ConfigContext.getConfigInstance(S3CloudSharedConfig.class);
        }
        return config;
    }

    @Identifier(value = "region", name = "Region", description = "Plugin-S3Cloud_Shared_Region", status = Status.SHOW)
    private Conf<String> region = Holders.simple(StringUtils.EMPTY);

    // 地域节点配置信息
    @Identifier(value = "endpoint", name = "Endpoint", description = "Plugin-S3Cloud_Shared_End_Point", status = Status.SHOW)
    private Conf<String> endpoint = Holders.simple(StringUtils.EMPTY);

    @Identifier(value = "accessKeyId", name = "AccessKeyId", description = "Plugin-S3Cloud_Shared_AccessKeyId", status = Status.SHOW)
    private Conf<String> accessKeyId = Holders.simple(StringUtils.EMPTY);

    @Identifier(value = "accessKeySecret", name = "AccessKeySecret", description = "Plugin-S3Cloud_Shared_AccessKeySecret", status = Status.SHOW)
    private Conf<String> accessKeySecret = Holders.simple(StringUtils.EMPTY);

    public String getRegion() {
        return region.get();
    }

    public void setRegion(String region) {
        this.region.set(region);
    }

    /**
     * 获取地域信息
     * @return 地域信息
     */
    public String getEndpoint() {
        return endpoint.get();
    }

    /**
     * 设置地域信息
     * @param endpoint S3存储所在的地域
     */
    @Focus(id="com.fanruan.cloud.s3.shared.config", text = "Plugin-S3Cloud_Shared_Config")
    public void setEndpoint(String endpoint) {
        this.endpoint.set(endpoint);
    }

    /**
     * 获取账号
     * @return 账号
     */
    public String getAccessKeyId() {
        return accessKeyId.get();
    }

    /**
     * 设置账号
     * @param accessKeyId 账号
     */
    public void setAccessKeyId(String accessKeyId) {
        this.accessKeyId.set(accessKeyId);
    }

    /**
     * 获取密码
     * @return 密码
     */
    public String getRealAccessKeySecret() {
        return accessKeySecret.get();
    }

    /**
     * 获取密码
     * @return 密码
     */
    public String getAccessKeySecret() {
        return PASSWORD_HOLDER;
    }

    /**
     * 设置密码
     * @param accessKeySecret 密码
     */
    public void setAccessKeySecret(String accessKeySecret) {
        if (PASSWORD_HOLDER.equals(accessKeySecret)) {
            return;
        }
        this.accessKeySecret.set(accessKeySecret);
    }

    @Override
    public Object clone() throws CloneNotSupportedException {
        S3CloudSharedConfig cloned = (S3CloudSharedConfig) super.clone();
        cloned.region = (Conf<String>) region.clone();
        cloned.endpoint = (Conf<String>) endpoint.clone();
        cloned.accessKeyId = (Conf<String>) accessKeyId.clone();
        cloned.accessKeySecret = (Conf<String>) accessKeySecret.clone();
        return cloned;
    }
}
